#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
scf
~~~~~~~~~~~~~~
This module is exception of scf api
"""

import sys


class TencentServerlessSDKException(Exception):
    """tencentserverlessapi sdk 异常类"""

    def __init__(self, code=None, message=None, request_id=None,
                 response=None, stack_trace=''):
        self.code = code
        self.message = message
        self.request_id = request_id
        self.response = response
        self.stack_trace = stack_trace

    def __str__(self):
        message = "[TencentServerlessSDKException] \ncode:%s \nmessage:%s " \
            "\nstack_trace:%s \nrequestId:%s" % (self.code, self.message,
                                                 self.stack_trace, self.request_id)
        if sys.version_info[0] < 3 and isinstance(message, unicode):
            return message.encode("utf8")
        else:
            return message

    def get_code(self):
        """
        get_code
        ~~~~~~~~~~~~~~
        :return: code
        """
        return self.code

    def get_message(self):
        """
        get_message
        ~~~~~~~~~~~~~~
        :return: message
        """
        return self.message

    def get_request_id(self):
        """
        get_message
        ~~~~~~~~~~~~~~
        :return: request_id
        """
        return self.request_id

    def get_stack_trace(self):
        """
        get_message
        ~~~~~~~~~~~~~~
        :return: stack_trace
        """
        return self.stack_trace

    def get_response(self):
        """
        get_message
        ~~~~~~~~~~~~~~
        :return: response
        """
        return self.response



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
scf
~~~~~~~~~~~~~~
This module implements scf api
"""

import os
import json
from tencentserverless.exception import TencentServerlessSDKException
from tencentcloud.common import credential
from tencentcloud.common.exception.tencent_cloud_sdk_exception import TencentCloudSDKException
from tencentcloud.scf.v20180416 import scf_client
from tencentcloud.common.profile.http_profile import HttpProfile
from tencentcloud.common.profile.client_profile import ClientProfile


class Client(object):
    """
    Client
    ~~~~~~~~~~~~~~
    This class default a client for scf api
    """
    def __init__(self,
                 region=None,
                 secret_id=None,
                 secret_key=None,
                 token=None):

        self.region = region if region is not None \
            else os.environ.get("TENCENTCLOUD_REGION", "ap-guangzhou")

        self.secret_id = secret_id if secret_id is not None \
            else os.environ.get("TENCENTCLOUD_SECRETID", None)

        self.secret_key = secret_key if secret_key is not None \
            else os.environ.get("TENCENTCLOUD_SECRETKEY", None)

        self.token = token if token is not None \
            else os.environ.get("TENCENTCLOUD_SESSIONTOKEN", None)

        self.env = os.environ.get("TENCENTCLOUD_RUNENV", None)

        self.endpoint = "scf.internal.tencentcloudapi.com" if self.env == "SCF" \
            else "scf.tencentcloudapi.com"

        cred = credential.Credential(self.secret_id, self.secret_key, self.token)
        profile = ClientProfile(httpProfile=HttpProfile(reqTimeout=300, keepAlive=True))
        self.client = scf_client.ScfClient(cred, self.region, profile=profile)

    def invoke(self, function_name,
               namespace="default",
               log_type="None",
               qualifier="$LATEST",
               invocation_type="RequestResponse",
               data=None):

        client_context = None
        if data:
            client_context = json.dumps(data)
        action_params = {
            "Namespace": namespace,
            "LogType": log_type,
            "ClientContext": client_context,
            "Qualifier": qualifier,
            "InvocationType": invocation_type,
            "FunctionName": function_name,
        }
        res = json.loads(self.client.call("Invoke", action_params))
        if "Error" in res["Response"]:
            raise TencentCloudSDKException(code=res["Response"]["Error"]["Code"],
                                           message=res["Response"]["Error"]["Message"],
                                           requestId=res["Response"]["RequestId"])
        elif "InvokeResult" in res["Response"]["Result"]:
            if res["Response"]["Result"]["InvokeResult"] == 1:
                exceptions = json.loads(res["Response"]["Result"]["RetMsg"])
                stack_trace = exceptions["stackTrace"] if "stackTrace" in exceptions else ""
                raise TencentServerlessSDKException(code=exceptions["errorCode"],
                                                    message=exceptions["errorMessage"],
                                                    response=res, stack_trace=stack_trace,
                                                    request_id=res["Response"]['RequestId'])
            elif res["Response"]["Result"]["InvokeResult"] == -1:
                raise TencentServerlessSDKException(code=-1,
                                                    message=res["Response"]["Result"]["RetMsg"],
                                                    response=res,
                                                    request_id=res["Response"]['RequestId'])
            else:
                return res["Response"]["Result"]["RetMsg"]
        else:
            raise TencentServerlessSDKException(code=-1, message='Internal server error',
                                                response=res,
                                                request_id=res["Response"]['RequestId'])


def invoke(function_name,
           region=None,
           secret_id=None,
           secret_key=None,
           token=None,
           namespace="default",
           log_type="None",
           qualifier="$LATEST",
           invocation_type="RequestResponse",
           data=None):

    client = Client(region=region, secret_id=secret_id, secret_key=secret_key, token=token)
    return client.invoke(function_name=function_name, namespace=namespace, log_type=log_type,
                         qualifier=qualifier, invocation_type=invocation_type, data=data)



import os
import io
import setuptools

def read(*filenames, **kwargs):
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', os.linesep)
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)

def read_requirements(req='requirements.txt'):
    content = read(os.path.join(req))
    return [line for line in content.split(os.linesep)
            if not line.strip().startswith('#')]

setuptools.setup(
    name='tencentserverless',
    version='0.1.0',
    keywords='scf',
    description='This is callFunction for SCF.',
    author='Tencent Cloud',
    author_email='qcloud_middleware@qq.com',
    url='https://github.com/alanoluo/tencent-serverless-python.git',
    packages=setuptools.find_packages(),
    install_requires=read_requirements('requirements.txt'),
    license='Apache License 2.0'
)
